import cmor._cmor
atts = """
CMOR_MAX_STRING
CMOR_MAX_ELEMENTS
CMOR_MAX_AXES
CMOR_MAX_VARIABLES
CMOR_MAX_GRIDS
CMOR_MAX_DIMENSIONS
CMOR_MAX_ATTRIBUTES
CMOR_MAX_ERRORS
CMOR_MAX_TABLES
CMOR_MAX_GRID_ATTRIBUTES
CMOR_QUIET
CMOR_EXIT_ON_MAJOR
CMOR_EXIT
CMOR_EXIT_ON_WARNING
CMOR_VERSION_MAJOR
CMOR_VERSION_MINOR
CMOR_VERSION_PATCH
CMOR_CF_VERSION_MAJOR
CMOR_CF_VERSION_MINOR
CMOR_WARNING
CMOR_NORMAL
CMOR_CRITICAL
CMOR_N_VALID_CALS
CMOR_PRESERVE
CMOR_APPEND
CMOR_REPLACE
CMOR_PRESERVE_3
CMOR_APPEND_3
CMOR_REPLACE_3
CMOR_PRESERVE_4
CMOR_APPEND_4
CMOR_REPLACE_4
GLOBAL_ATT_HISTORYTMPL
CMOR_DEFAULT_HISTORY_TEMPLATE
"""

for att in atts.split():
    attnm = att
    exec("%s = cmor._cmor.getCMOR_defaults_include('%s')" % (att, att))



# -*- coding: utf-8 -*-
'''
Created on Fri Feb 19 11:33:52 2016

@author: Denis Nadeau LLNL, Charles Doutriaux LLNL
'''

from cmip6_cv import cmor_const
import numpy
import os
from cmip6_cv import _cmip6_cv
import signal
import six


def sig_handler(signum, frame):
    os.kill(os.getpid(), signal.SIGABRT)


signal.signal(signal.SIGTERM, sig_handler)
signal.signal(signal.SIGINT, sig_handler)

try:
    import cdtime
    has_cdtime = True
except BaseException:
    has_cdtime = False

# try:
    #import cdms2
    #has_cdms2 = True
# except BaseException:
    #has_cdms2 = False

# try:
    #import MV2
    #has_MV2 = True
# except BaseException:
    #has_MV2 = False

try:
    import numpy.oldnumeric.ma.MaskedArray
    has_oldma = True
except BaseException:
    has_oldma = False


def _to_numpy(vals, message):
    if isinstance(vals, (list, tuple)):
        vals = numpy.ascontiguousarray(vals)
    elif not isinstance(vals, numpy.ndarray):
        try:
            vals = numpy.ascontiguousarray(vals.filled())
        except BaseException:
            raise Exception(
                "Error could not convert %s to a numpy array" %
                message)

    return vals


def setup(inpath='.',
          set_verbosity=cmor_const.CMOR_NORMAL, exit_control=cmor_const.CMOR_NORMAL,
          logfile=None, create_subdirectories=1):
    """
    Usage cmor_setup(inpath='.',netcdf_file_action=cmor.CMOR_PRESERVE,
                     set_verbosity=cmor.CMOR_NORMAL,exit_control=cmor.CMOR_NORMAL)
    Where:
    path:                  Alternate directory where to find tables if not in current directory
    set_verbosity:         CMOR_QUIET or CMOR_NORMAL
    exit_control:          CMOR_EXIT_ON_WARNING, CMOR_EXIT_ON_MAJOR, CMOR_NORMAL
    create_subdirectories: 1 to create subdirectories structure, 0 to dump files directly where cmor_dataset tells to
"""
    if not isinstance(exit_control, int) or not exit_control in [cmor_const.CMOR_EXIT_ON_WARNING,
                                                                 cmor_const.CMOR_EXIT_ON_MAJOR,
                                                                 cmor_const.CMOR_NORMAL]:
        raise Exception("exit_control must an integer valid values are: CMOR_EXIT_ON_WARNING, "
                        "CMOR_EXIT_ON_MAJOR, CMOR_NORMAL")

    if not isinstance(set_verbosity, int) or not set_verbosity in [
            cmor_const.CMOR_QUIET, cmor_const.CMOR_NORMAL]:
        raise Exception(
            "set_verbosity must an integer valid values are: CMOR_QUIET, CMOR_NORMAL")

    if not isinstance(inpath, str) and not os.path.exists(inpath):
        raise Exception("path must be a Valid path")
    if logfile is None:
        logfile = ""

    if create_subdirectories not in [0, 1]:
        raise Exception("create_subdirectories must be 0 or 1")
    CMOR_PRESERVE = 10
    return(_cmip6_cv.setup(inpath, CMOR_PRESERVE, set_verbosity, exit_control, logfile, create_subdirectories))


def load_table(table):
    """ loads a cmor table
    Usage:
    load_table(table)
    """
    if not isinstance(table, six.string_types):
        raise Exception("Error, must pass a string")
#     if not os.path.exists(table):
#         raise Exception, "Error, the table you specified (%s) does not exists" % table
    return _cmip6_cv.load_table(table)


def set_table(table):
    if not isinstance(table, int):
        raise Exception("error you need to pass and integer as the table id")
    return(_cmip6_cv.set_table(table))


def close(var_id=None, file_name=False, preserve=False):
    """ Close CMOR variables/file
    Usage:
      cmor.close(varid=None)
    Where:
      var_id: id of variable to close, if passing None, means close every open ones.
      [file_name] True/False (default False) if True: return name of the file just closed,
                  works only if var_id is not None
      [preserve] True/False (default False) if True: close the file but preserve the var
                  definition in CMOR to write more data with this variable (into a new file)
      """
    if var_id is not None and not isinstance(var_id, int):
        raise Exception("Error var_id must be None or a integer")

    if (preserve is False):
        if (file_name is False):
            return(_cmip6_cv.close(var_id, 0, 0))
        else:
            return(_cmip6_cv.close(var_id, 1, 0))
    else:
        if (file_name is False):
            return(_cmip6_cv.close(var_id, 0, 1))
        else:
            return(_cmip6_cv.close(var_id, 1, 1))


def set_cur_dataset_attribute(name, value):
    """Sets an attribute onto the current cmor dataset
    Usage:
      cmor.set_cur_dataset_attribute(name,value)
    Where:
      name: is the name of the attribute
      value: is the value for this attribute
    """
    if value is None:
        val = ""
    else:
        val = str(value)
    return(_cmip6_cv.set_cur_dataset_attribute(name, val))


def has_cur_dataset_attribute(name):
    """determines if the current cmor dataset has an attribute
    Usage:
      cmor.has_cur_dataset_attribute(name)
    Where:
      name: is the name of the attribute
    Returns True if the dataset has the attribute, False otherwise
    """
    test = _cmip6_cv.has_cur_dataset_attribute(name)
    if test == 0:
        return True
    else:
        return False


def get_cur_dataset_attribute(name):
    """Gets an attribute from the current cmor dataset
    Usage:
      cmor.get_cur_dataset_attribute(name)
    Where:
      name: is the name of the attribute
    Returns none if attribute is non-existant
    """
    if has_cur_dataset_attribute(name):
        return _cmip6_cv.get_cur_dataset_attribute(name)
    else:
        return None


def set_variable_attribute(var_id, name, value):
    """Sets an attribute onto a cmor variable
    Usage:
      cmor.set_variable_attribute(var_id,name,value)
    Where:
      var_id: is cmor variable id
      name  : is the name of the attribute
      value : is the value for this attribute
    """
    if value is None:
        val = ""
    else:
        val = str(value)
    return _cmip6_cv.set_variable_attribute(var_id, name, val)


def has_variable_attribute(var_id, name):
    """determines if the a cmor variable has an attribute
    Usage:
      cmor.het_variable_attribute(name)
    Where:
      var_id: is cmor variable id
      name: is the name of the attribute
    Returns True if the dataset has the attribute, False otherwise
    """
    test = _cmip6_cv.has_variable_attribute(var_id, name)
    if test == 0:
        return True
    else:
        return False


def list_variable_attributes(var_id):
    """List all attribute from a cmor variable
    Usage:
      cmip6_cv.list_variable_attribute(var_id)
    Where:
      var_id: is cmor variable id
    Returns none if the variable as no attribute
    """
    var_list = _cmip6_cv.list_variable_attributes(var_id)
    return var_list


def get_variable_attribute(var_id, name):
    """Gets an attribute from a cmor variable
    Usage:
      cmor.get_variable_attribute(name)
    Where:
      var_id: is cmor variable id
      name: is the name of the attribute
    Returns none if attribute is non-existant
    """
# print 'In there asking for attribute: ',name,'on var',var_id
    if has_variable_attribute(var_id, name):
        # print 'Seems to have it',var_id,name
        return _cmip6_cv.get_variable_attribute(var_id, name)
    else:
        return None


def check_institution(table_id):
    '''
        Verify institution against Control Vocabulary files
        Usage:
          cmip6_cv.set_institution(table_id)
        Where:
          table_id: is the table id returned by load_table()
        Returns 0 on success
    '''
    ierr = _cmip6_cv.set_institution(table_id)

    return(ierr)


def check_sourceID(table_id):
    '''
      Validate source and source id against Control Vocabulary file.
      Usage:
        cmip6_cv.check_sourceID(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_sourceID(table_id)
    return(ierr)


def check_filename(table_id, var_name, calendar, timeunits, infile):
    '''
      Validate filename with timestamp for current variable and file

      Usage:
        cmip6_cv.check_filename(table_id, var_id)
      Where:
        table_id is the table id returned by load_table()
        var_name is the variable name
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_filename(table_id, var_name,
                                    calendar, timeunits, infile)
    return(ierr)


def check_experiment(table_id):
    '''
      Validate Experiment and Experiement_id against Control Vocabulary file.

      If required global attributes are not all set, warning will be displayed.

      Usage:
        cmip6_cv.check_experiement(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_experiment(table_id)
    return(ierr)


def check_grids(table_id):
    '''
      Validate grid and grid_resolution against Control Vocabulary file.

      Usage:
        cmip6_cv.check_grids(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_grids(table_id)
    return(ierr)


def check_requiredattributes(table_id):
    '''
      Validate all required attributes against Control Vocabulary file.

      Usage:
        cmip6_cv.check_requiredattributes(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_gblattributes(table_id)
    return(ierr)


def check_subExpID(table_id):
    '''
      Validate that sub_experiment ind sub_experiment_id are set to appropriate value
      as defined in the CV file.

      Usage:
        cmip6_cv.check_subExpID(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_subExpID(table_id)
    return(ierr)


def check_parentExpID(table_id):
    '''
      Validate that parent_experiement is set to appropriate value
      if parent is set to "no parent" validate that other related
      attributes are set to "no parent"

      if parent is set to any string, validate all related attributes.

      Usage:
        cmip6_cv.check_parentExpID(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_parentExpID(table_id)
    return(ierr)


def check_ISOTime():
    '''
      Validate that creation attribute contains time
      in ISO format YYYY-MM-DDTHH:MM:SS.

      Usage:
        cmip6_cv.check_ISOtime()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_ISOTime()
    return(ierr)


def setup_variable(name, units, missing, imissing, startime,
                   endtime, startimebnds, endtimebnds):
    '''
    Create variable  attributes from the table loaded by load_table.

    Usage:
        cmip6_cv.check_variable( name, units, missing )
    Where:
        name is the variable name to validate
        units are the variable units
        missing is the missing floating-point value for this variable.
        imissing is the missing integer value for this variable.
        startime: time value for first part of timestap -- time[0]
        endtime: time value for last part of timestap -- time[-1]
        startimebnds: time bound value for first part of timestap -- time_bnds[0]
        endtimebnds: time bound value for last part of timestap -- time_bnds[-1]
    return: variable_id on success
           -1 on failure
    '''
    ierr = _cmip6_cv.setup_variable(
        name,
        units,
        missing,
        imissing,
        startime,
        endtime,
        startimebnds,
        endtimebnds)
    return(ierr)


def reset_CV_Error():
    '''
    set CV_Error to 0
    '''
    _cmip6_cv.reset_CV_Error()
    return


def get_CV_Error():
    '''
    return 0 if no error was set by CV
    '''
    ierr = _cmip6_cv.get_CV_Error()
    return(ierr)


def set_CV_Error():
    '''
    set CV_ERROR to 1.
    '''
    _cmip6_cv.set_CV_Error()
    return


def check_furtherinfourl(table_id):
    '''
      Validate further info URL attribute using REGEX found in Control Vocabulary file.

      Usage:
        cmip6_cv.furtherinfourl(table_id)
      Where:
        table_id is the table id returned by load_table()
      Return 0 on success
    '''
    ierr = _cmip6_cv.check_furtherinfourl(table_id)
    return(ierr)



from __future__ import print_function
import cmor
import numpy
import cdtime


def cmor_initialisation():
    cmor.setup(inpath='/git/cmip5-cmor-tables/Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', '360_day',
                 institute_id='ukmo',
                 model_id='HadCM3',
                 history='some global history',
                 forcing='N/A',
                 parent_experiment_id='N/A',
                 parent_experiment_rip='N/A',
                 branch_time=0.,
                 contact='bob',
                 outpath='./out')


def setup_data():
    axes = [{'table_entry': 'time1',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'site',
             'units': '',
             'coord_vals': [0]},
            {'table_entry': 'hybrid_height',
             'units': 'm',
             'coord_vals': range(2),
             'cell_bounds': [[x - 0.5, x + 0.5] for x in range(2)],
             },
            ]

    values = numpy.array([0.5, 0.5], numpy.float32)
    return values, axes


def cmor_define_and_write(values, axes):
    table = 'CMIP5_cfSites'
    cmor.load_table(table)

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    igrid = cmor.grid([axis_ids[1]], [0.], [0.])
    cmor.zfactor(axis_ids[2], 'b', axis_ids=[axis_ids[2]],
                 zfactor_values=range(2),
                 zfactor_bounds=[[x - 0.5, x + 0.5] for x in range(2)])

    cmor.zfactor(axis_ids[2], 'orog', 'm', axis_ids=[igrid],
                 zfactor_values=[0])

    ids_for_var = [axis_ids[0], igrid, axis_ids[2]]
    varid = cmor.variable('tnhus',
                          's-1',
                          ids_for_var,
                          history='variable history',
                          missing_value=-99,
                          )

    for time in [x * 1800. / 86400 for x in range(48)]:
        time += 1. / 3600. / 24.
        tr = cdtime.reltime(time, axes[0]["units"])
        print("Writing: %.03f" % time, "|", tr.tocomp(cdtime.Calendar360), "|", tr.tocomp())
        cmor.write(varid, values, time_vals=[time])
    return varid


def main():

    cmor_initialisation()
    values, axes = setup_data()
    varid = cmor_define_and_write(values, axes)
    fname = cmor.close(varid, file_name=True)
    print("Done:", fname)


if __name__ == '__main__':

    main()



#!/usr/bin/env python
from __future__ import print_function
import cmor
import numpy


def main():

    cmor.setup(inpath='/git/cmip5-cmor-tables/Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', 'noleap',
                 institute_id='ukmo',
                 model_id='HadCM3',
                 history='some global history',
                 forcing='N/A',
                 parent_experiment_id='N/A',
                 parent_experiment_rip='N/A',
                 branch_time=0,
                 contact='brian clough')

    table = 'CMIP5_Oclim'
    cmor.load_table(table)
    axes = [{'table_entry': 'time2',
             'units': 'days since 1861',
             'coord_vals': [48925.5, 48955, 48984.5, 49015, 49045.5, 49076, 49106.5, 49137.5, 49168, 49198.5, 49229, 49259.5],
             'cell_bounds': [[45625, 52591],
                             [45656, 52619, ],
                             [45684, 52650, ],
                             [45715, 52680, ],
                             [45745, 52711, ],
                             [45776, 52741, ],
                             [45806, 52772, ],
                             [45837, 52803, ],
                             [45868, 52833, ],
                             [45898, 52864, ],
                             [45929, 52894, ],
                             [45959, 52925]],
             },
            {'table_entry': 'depth_coord',
             'units': 'm',
             'coord_vals': [500, 1000.],
             'cell_bounds': [0., 750., 1200.]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    axis_ids = list()
    for axis in axes:
        print('doing:', axis)
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    for var, units, value in (('tnpeot', 'W m-2', 274),):
        values = numpy.ones(
            (len(
                axes[0]['coord_vals']), len(
                axes[1]['coord_vals']))) * value
        varid = cmor.variable(var,
                              units,
                              axis_ids,
                              history='variable history',
                              missing_value=-99
                              )
        print(values.shape)

        cmor.write(varid, values)  # , ntimes_passed=1)

    cmor.close()


if __name__ == '__main__':

    main()



import os
import sys
from testsrunner import TestRunnerBase


test_suite_name = 'cmor'

workdir = os.getcwd()

runner = TestRunnerBase(test_suite_name)
ret_code = runner.run(workdir)
sys.exit(ret_code)



import cmor
import numpy

error_flag = cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

error_flag = cmor.dataset_json("Test/CMOR_input_example.json")


# creates 1 degree grid
nlat = 18
nlon = 36
alats = numpy.arange(180) - 89.5
bnds_lat = numpy.arange(181) - 90
alons = numpy.arange(360) + .5
bnds_lon = numpy.arange(361)
cmor.load_table("Tables/CMIP6_Amon.json")
error_flag = cmor.close()



from __future__ import print_function
import cmor
import numpy
import os
import sys

specs = {"CLOUD": {"convert": [2.0, 0.], "units": "%", "entry": "cl", "positive": ""},
         "U": {"convert": [1., -40.], "units": "m s-1", "entry": "ua", "positive": ""},
         "T": {"convert": [1., 220], "units": "K", "entry": "ta", "positive": ""},
         "LATENT": {"convert": [4., 0.], "units": "W m-2", "entry": "hfls", "positive": "down"},
         "TSURF": {"convert": [1., -15.], "units": "Celsius", "entry": "tas", "positive": ""},
         "PSURF": {"convert": [3., 900.], "units": "hPa", "entry": "ps", "positive": ""},
         "SOIL_WET": {"convert": [2., 0.], "units": "kg m-2", "entry": "mrsos", "positive": ""},
        "MC": {"convert": [2., 0.], "units": "kg m-2 s-1", "entry": "mc", "positive": "up"},
         }


def read_coords(nlats, nlons):
    alons = (360.*numpy.arange(nlons)/nlons+.5) - 180.
    blons = numpy.zeros((nlons, 2), dtype=float)
    blons[:, 0] = 360.*numpy.arange(nlons)/nlons - 180.
    blons[:, 1] = 360.*numpy.arange(1, nlons+1)/nlons - 180.

    alats = (180.*numpy.arange(nlats)/nlats+.5) - 90.
    blats = numpy.zeros((nlats, 2), dtype=float)
    blats[:, 0] = 180.*numpy.arange(nlats)/nlats - 90.
    blats[:, 1] = 180.*numpy.arange(1, nlats+1)/nlats - 90.

    levs = numpy.array([1., 5., 10, 20, 30, 50, 70, 100, 150,
                        200, 250, 300, 400, 500, 600, 700, 850, 925, 1000])

    return alats, blats, alons, blons, levs


def read_time(index, refyear=2015., monthdays=30., yeardays=360.):
    time0 = (refyear - 1850.) * yeardays
    time = time0 + (index + 0.5) * monthdays
    b1 = time0 + (index) * monthdays
    b2 = time0 + (index+1) * monthdays
    return numpy.array([time]), numpy.array([b1, b2])


def read_3d_input_files(index, varname, shape):

    print("3d shape",shape)
    field = numpy.zeros(shape, dtype=numpy.float32)
    factor, offset = specs[varname]["convert"]
    for i in range(field.shape[2]):  # lon
        for j in range(field.shape[1]):  # lat
            for k in range(field.shape[0]):  # levs
                if i == 0:
                    field[k, j, 0] = ((k)*3 + (j)*16 + index) * \
                        factor + offset + 0.225
                else:
                    field[k, j, i] = ((k)*3 + (j)*16 + index +
                                      4*i-2)*factor + offset + 0.225
    return field


def read_2d_input_files(index, varname, shape):
    sh = list(shape)
    sh.insert(0, 1)
    return read_3d_input_files(index, varname, sh)[0]


# PARAMETERS

ntimes = 2
lon = 4
lat = 3
plev = 19
lev = 5

p0 = 1.e3
a_coeff = [0.1, 0.2, 0.3, 0.22, 0.1]
b_coeff = [0.0, 0.1, 0.2, 0.5, 0.8]

a_coeff_bnds = [0., .15, .25, .25, .16, 0.]
b_coeff_bnds = [0., .05, .15, .35, .65, 1.]


varin3d = ["MC",]

n3d = len(varin3d)

alats, bnds_lat, alons, bnds_lon, plevs = read_coords(lat, lon)

print(alats[:2], alats[-2:])
print(bnds_lat[0], bnds_lat[-1])
print(alons[:2], alons[-2:])
print(bnds_lon[0], bnds_lon[-1])
print(plevs)

pth = os.path.expanduser("~/Karl")
pth = os.getcwd()
ierr = cmor.setup(inpath=os.path.join(pth, "Test"),
                  netcdf_file_action=cmor.CMOR_REPLACE) #, logfile="CMOR.log")
print("ERR:", ierr)
ierr = cmor.dataset_json(os.path.join(pth, "Test", "CMOR_input_example.json"))
print("ERR:", ierr)

ierr = cmor.load_table(os.path.join(pth, 'Tables', 'CMIP6_Amon.json'))
print("ERR:", ierr)
ilat = cmor.axis(
    table_entry='latitude',
    units='degrees_north',
    length=lat,
    coord_vals=alats,
    cell_bounds=bnds_lat)
print("ILAT:", ilat)
print(lon, alons, bnds_lon)
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=alons,
    units='degrees_east',
    cell_bounds=bnds_lon)

print("ILON:", ilon)


itim = cmor.axis(
    table_entry="time",
    units="days since 1850",
    length=ntimes)

print("ITIME:",itim)
zlevs = numpy.array([.1, .3, .55, .7, .9])
zlev_bnds = numpy.array([0., .2, .42, .62, .8, 1.])

ilev_half = cmor.axis(table_entry='standard_hybrid_sigma_half',
                 units='1',
                 coord_vals=zlevs, cell_bounds=zlev_bnds)
print("ILEVL half:",ilev_half)

cmor.zfactor(zaxis_id=ilev_half, zfactor_name='p0', units='hPa', zfactor_values=p0)
print("p0 1/2")
cmor.zfactor(zaxis_id=ilev_half, zfactor_name='b_half', axis_ids=[ilev_half, ],
             zfactor_values=b_coeff, zfactor_bounds=b_coeff_bnds)
print("b 1/2")
cmor.zfactor(zaxis_id=ilev_half, zfactor_name='a_half', axis_ids=[ilev_half, ],
             zfactor_values=a_coeff, zfactor_bounds=a_coeff_bnds)
print("a 1/2")
ps_var = cmor.zfactor(zaxis_id=ilev_half, zfactor_name='ps',
             axis_ids=[ilon, ilat, itim], units='hPa')
print("ps 1/2")

var3d_ids = []
for m in varin3d:
    print("3d VAR:",m)
    var3d_ids.append(
        cmor.variable(table_entry=specs[m]["entry"],
                      units=specs[m]["units"],
                      axis_ids=[itim, ilev_half, ilat, ilon],
                      missing_value=1.e28,
                      positive=specs[m]["positive"],
                      original_name=m)
    )
print("Ok now writing",var3d_ids, ntimes)
for index in range(ntimes):
    tim_array, bnds_tim = read_time(index)
    for i, varname in enumerate(varin3d):
        data = read_3d_input_files(index, varname,(lev,lat,lon))
        print(data.shape, data)
        print(tim_array, bnds_tim)
        cmor.write(var_id=var3d_ids[i], data=data, ntimes_passed=1,
                   time_vals=tim_array, time_bnds=bnds_tim)
        print("Passed write")
        # PSURF
        data = read_2d_input_files(index,"PSURF",(lat,lon))
        cmor.write(var_id=ps_var, data=data, ntimes_passed=1, time_vals=tim_array, time_bnds=bnds_tim, store_with=var3d_ids[i])
cmor.close()


from __future__ import print_function
import cmor
import numpy

nlat = 90
nlon = 180


def mywrite(data=None, time_vals=None, append_to=None, cell_bounds=None):
    breq = "100000. 80000. 80000. 68000. 68000. 56000. 56000. 44000. 44000. 31000. 31000. 18000. 18000.  0.".split()

    bnds_req = []
    for b in breq:
        bnds_req.append(float(b))

    bnds_req = numpy.array(bnds_req)
    bnds_req.shape = (7, 2)

    print(bnds_req[-2], bnds_req.shape)

    levs = []

    for b in bnds_req:
        levs.append((b[0] + b[1]) / 2.)

    levs = numpy.array(levs)

    print(levs)

    ipth = "Test"
    if append_to is None:
        mode = cmor.CMOR_REPLACE
    else:
        mode = cmor.CMOR_APPEND
    print('Mode in python:', mode)
    cmor.setup(inpath=ipth,
               set_verbosity=cmor.CMOR_NORMAL,
               netcdf_file_action=mode,
               logfile=None)

    cmor.dataset_json("Test/CMOR_input_example.json")

    cmor.load_table("Tables/CMIP6_Amon.json")

    dlat = 180 / nlat
    dlon = 360. / nlon
    lats = numpy.arange(-90 + dlat / 2., 90, dlat)
    bnds_lat = numpy.arange(-90, 90 + dlat, dlat)
    lons = numpy.arange(0 + dlon / 2., 360., dlon) - 180.
    bnds_lon = numpy.arange(0, 360. + dlon, dlon) - 180.

    plevs = numpy.array([100000., 92500., 85000., 70000.,
                         60000., 50000., 40000., 30000., 25000., 20000.,
                         15000., 10000., 7000., 5000., 3000., 2000., 1000., 500., 100.])

    itim = cmor.axis(table_entry='time', units='month since 2008')
    #itim = cmor.axis(table_entry='time',units='month since 2008',coord_vals=numpy.arange(0,12,1))
    ilat = cmor.axis(
        table_entry='latitude',
        coord_vals=lats,
        units='degrees_north',
        cell_bounds=bnds_lat)
    ilon = cmor.axis(
        table_entry='longitude',
        coord_vals=lons,
        units='degrees_east',
        cell_bounds=bnds_lon)
    print('so far', itim, ilat, ilon)
    ilev = cmor.axis(table_entry="plev19", coord_vals=plevs, units="Pa")

    iv = cmor.variable(
        table_entry='ta', axis_ids=numpy.array(
            (itim, ilev, ilat, ilon)), units='K')

    # cmor.write(iv,data)
    if append_to is None:
        print('time:', time_vals)
        print('bnds:', cell_bounds)
        # ,file_suffix="with-appending")
        cmor.write(iv, data, time_vals=time_vals, time_bnds=cell_bounds)
    else:
        print('Ok writing with a suffix', append_to)
        cmor.write(
            iv,
            data,
            time_vals=time_vals,
            file_suffix=append_to,
            time_bnds=cell_bounds)
        print('and back')
    file = cmor.close(iv, file_name=True)
    print('Ok dumped to:', file)
    cmor.close()
    return file


ntime = 12
data = numpy.random.random((ntime, 19, nlat, nlon)) + 280.

f1 = mywrite(data=data[:6], time_vals=numpy.arange(
    0, 6, 1), cell_bounds=numpy.arange(0, 7, 1))
print('First part: ', f1)
f2 = mywrite(data=data[6:], time_vals=numpy.arange(
    6, 12, 1), cell_bounds=numpy.arange(6, 13, 1), append_to=f1)
print(f2)



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import sys
import os
import tempfile
import time

try:
    import cdms2
    cdms2.setNetcdfShuffleFlag(0)
    cdms2.setNetcdfDeflateFlag(0)
    cdms2.setNetcdfDeflateLevelFlag(0)
except BaseException:
    print("This test code needs a recent cdms2 interface for i/0")
    sys.exit()


class TestCase(unittest.TestCase):

    def testCMIP6_defaultmissinginteger(self):
        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        try:
            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
            cmor.dataset_json("Test/CMOR_input_example.json")
            # ------------------------------------------
            # load Omon table and create masso variable
            # ------------------------------------------
            cmor.load_table("CMIP6_Omonmissing.json")

            itime = cmor.axis(table_entry="time", units='months since 2010',
                              coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                              cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))

            ivar = cmor.variable(
                table_entry="massobad",
                axis_ids=[itime],
                type=numpy.dtype('int32').char,
                units='kg')

            data = numpy.random.random(5).astype('int32')
            # set fill_value
            data[0]=23;
            data[4]=23;

            for i in range(0, 5): 
                cmor.write(ivar, data[i:i])
            cmor.close()
        except BaseException:
            raise

        version =time.strftime("%Y%m%d")
        f=cdms2.open("CMIP6/CMIP6/ISMIP6/PCMDI/PCMDI-test-1-0/piControl-withism/r3i1p1f1/Omon/massoint/gn/v"+version+"/massoint_Omon_PCMDI-test-1-0_piControl-withism_r3i1p1f1_gn_201001-201005.nc")
        var=f['massoint']
        self.assertEqual(var.missing_value[0], -999)

    def testCMIP6missingvalue_integer(self):
        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        try:
            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
            cmor.dataset_json("Test/CMOR_input_example.json")
            # ------------------------------------------
            # load Omon table and create masso variable
            # ------------------------------------------
            cmor.load_table("CMIP6_Omonmissing.json")

            itime = cmor.axis(table_entry="time", units='months since 2010',
                              coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                              cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))

            ivar = cmor.variable(
                table_entry="massobad",
                axis_ids=[itime],
                type=numpy.dtype('int32').char,
                units='kg', missing_value=23)

            data = numpy.random.random(5).astype('int32')
            # set fill_value
            data[0]=23;
            data[4]=23;

            for i in range(0, 5):
                cmor.write(ivar, data[i:i])
            cmor.close()
        except BaseException:
            raise


        version =time.strftime("%Y%m%d")
        f=cdms2.open("CMIP6/CMIP6/ISMIP6/PCMDI/PCMDI-test-1-0/piControl-withism/r3i1p1f1/Omon/massoint/gn/v"+version+"/massoint_Omon_PCMDI-test-1-0_piControl-withism_r3i1p1f1_gn_201001-201005.nc")
        var=f['massoint']
        self.assertEqual(var.missing_value[0], -999)
        self.assertTrue(var[0].mask)
        self.assertEqual(var.dtype, numpy.dtype("int32"))

    def tearDown(self):
        import shutil
#        shutil.rmtree("./CMIP6")



if __name__ == '__main__':
    unittest.main()



import cmor
import numpy


def cmor_initialisation():
    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset_json("Test/CMOR_input_example.json")


def setup_data():
    axes = [{'table_entry': 'time1',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'site',
             'units': '',
             'coord_vals': [0]},
            {'table_entry': 'hybrid_height',
             'units': 'm',
             'coord_vals': numpy.array(range(2), dtype=numpy.float64),
             'cell_bounds': [[x - 0.5, x + 0.5] for x in range(2)],
             },
            ]

    values = numpy.array([0.5, 0.5], numpy.float32)
    return values, axes


def cmor_define_and_write(values, axes):
    table = 'CMIP6_CFsubhr.json'
    cmor.load_table(table)

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    igrid = cmor.grid([axis_ids[1]], [0.], [0.])
    cmor.zfactor(axis_ids[2], 'b', axis_ids=[axis_ids[2]],
                 zfactor_values=numpy.array(range(2), dtype=numpy.float64), 
                 zfactor_bounds=[[x - 0.5, x + 0.5] for x in range(2)])

    cmor.zfactor(axis_ids[2], 'orog', 'm', axis_ids=[igrid],
                 zfactor_values=[0.] )

    ids_for_var = [axis_ids[0], igrid, axis_ids[2]]
    varid = cmor.variable('tnhus',
                          's-1',
                          ids_for_var,
                          history='variable history',
                          missing_value=-99,
                          )

    for time in [x * 1800. / 86400 for x in range(48)]:
        cmor.write(varid, values, time_vals=[time])
    return varid


def main():

    cmor_initialisation()
    values, axes = setup_data()
    varid = cmor_define_and_write(values, axes)
    fname = cmor.close(varid, file_name=True)


if __name__ == '__main__':

    main()



from __future__ import print_function
from common import *
import cmor


cmor.setup(inpath="Test",
           set_verbosity=cmor.CMOR_NORMAL,
           netcdf_file_action=cmor.CMOR_REPLACE)

cmor.dataset_json("Test/metadata-template.json")
cmor.set_cur_dataset_attribute("calendar", "gregorian")
cmor.load_table("Tables/CMIP6_Amon.json")
ilat = cmor.axis(
    table_entry='latitude',
    units='degrees_north',
    length=lat,
    coord_vals=alats,
    cell_bounds=bnds_lat)
print("ILAT:", ilat)
print(lon, alons, bnds_lon)
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=alons,
    units='degrees_east',
    cell_bounds=bnds_lon)

itim = cmor.axis(
    table_entry="time",
    units="days since 1850",
    length=ntimes)

var2d_ids = []
for m in ["TSURF", ]:
    var2d_ids.append(
        cmor.variable(table_entry=specs[m]["entry"],
                      units=specs[m]["units"],
                      axis_ids=[itim, ilat, ilon],
                      missing_value=1.e28,
                      positive=specs[m]["positive"],
                      original_name=m)
    )

for index in range(ntimes):
    tim_array, bnds_tim = read_time(index)
    for i, varname in enumerate(["TSURF", ]):
        data = read_2d_input_files(index, varname, (lat, lon))
        print(data.shape, data)
        print(tim_array, bnds_tim)
        cmor.write(var_id=var2d_ids[i], data=data, ntimes_passed=1,
                   time_vals=tim_array, time_bnds=bnds_tim)
        print("Passed write")

cmor.close()



from __future__ import print_function
import cmor
import numpy
import sys
import os
from time import localtime, strftime
today = strftime("%Y%m%d", localtime())

try:
    import cdms2
    cdms2.setNetcdfShuffleFlag(0)
    cdms2.setNetcdfDeflateFlag(0)
    cdms2.setNetcdfDeflateLevelFlag(0)
except BaseException:
    print("This test code needs a recent cdms2 interface for i/0")
    sys.exit()


cmor.setup(
    inpath="Tables",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE_4,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
tables.append(cmor.load_table("CMIP6_chunking.json"))
print('Tables ids:', tables)


# read in data, just one slice
f = cdms2.open('data/tas_ccsr-95a.xml')
s = f("tas", time=slice(0, 12), squeeze=1)
ntimes = 12
varout = 'tas'

myaxes = numpy.arange(10)
myvars = numpy.arange(10)
myaxes[0] = cmor.axis(table_entry='latitude',
                      units='degrees_north',
                      coord_vals=s.getLatitude()[:], cell_bounds=s.getLatitude().getBounds())
myaxes[1] = cmor.axis(table_entry='longitude',
                      units='degrees_north',
                      coord_vals=s.getLongitude()[:], cell_bounds=s.getLongitude().getBounds())


myaxes[2] = cmor.axis(table_entry='time',
                      units=s.getTime().units,
                      coord_vals=s.getTime()[:], cell_bounds=s.getTime().getBounds())

pass_axes = [myaxes[2], myaxes[0], myaxes[1]]

myvars[0] = cmor.variable(table_entry=varout,
                          units=s.units,
                          axis_ids=pass_axes,
                          original_name=s.id,
                          history='no history',
                          comment='testing speed'
                          )


cmor.write(myvars[0], s.filled(), ntimes_passed=ntimes)
cmor.close()

#lcmor = os.stat("CMIP6/ISMIP6/PCMDI/PCMDI-test-1-0/piControl-withism/r11i1p1f1/Amon/tas/gr/v%s/tas_Amon_piControl-withism_PCMDI-test-1-0_r11i1p1f1_gr_197901-197912.nc"%(today))[6]



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import os
import sys
import tempfile

# ==============================
#  main thread
# ==============================


def run():
    unittest.main()


class TestCase(unittest.TestCase):

    def testCMIP6(self):

        # Create x and y coordinates and bounds.
        #
        coordVals = numpy.arange(-533750.0, 533750.0 +
                                 2500.0, 2500.0, numpy.float32)
        coordBnds = numpy.zeros((coordVals.shape[0], 2), numpy.float32)

        coordBnds[:, 0] = coordVals - 1250.0
        coordBnds[:, 1] = coordVals + 1250.0

        # Create longitude and latitude fields for a polar stereographic projection.
        #
        xgrid, ygrid = numpy.broadcast_arrays(
            numpy.expand_dims(
                coordVals, 0), numpy.expand_dims(
                coordVals, 1))

        rhogrid = numpy.sqrt(xgrid**2 + ygrid**2)
        cgrid = 2.0 * numpy.arctan((0.5 / 6378137.0) * rhogrid)
        latgrid = (180.0 / 3.141592654) * numpy.arcsin(numpy.cos(cgrid))
        longrid = (180.0 / 3.141592654) * numpy.arctan2(xgrid, -ygrid)

        # Set up CMOR with information from the CMOR config dictionary.
        #

        cmor.setup("Tables", netcdf_file_action=cmor.CMOR_REPLACE_4)

        # Create the output CMOR dataset using the output configuration.
        #
        cmor.dataset_json("Test/CMOR_input_example.json")

        # Load the grid table.
        #
        cmor_table_obj = cmor.load_table("CMIP6_grids.json")

        # Create ygre and xgre axes.
        #
        entry = {
            'table_entry': 'y',
            'units': 'm',
            'coord_vals': coordVals,
            'cell_bounds': coordBnds}

        axis_ygre = cmor.axis(**entry)

        entry = {
            'table_entry': 'x',
            'units': 'm',
            'coord_vals': coordVals,
            'cell_bounds': coordBnds}

        axis_xgre = cmor.axis(**entry)

        # Create the grid
        #
        grid_id = cmor.grid(
            axis_ids=[
                axis_ygre,
                axis_xgre],
            latitude=latgrid,
            longitude=longrid)

        # Set the CMOR grid mapping.
        #
        #    mapnm = 'polar_stereographic'
        param_dict = {
            'latitude_of_projection_origin': [90.0, 'degrees_north'],
            'straight_vertical_longitude_from_pole': [135.0, 'degrees_east'],
            'standard_parallel': [70.0, 'degrees_north'],
            'false_northing': [0.0, 'meters'],
            'false_easting': [0.0, 'meters']
        }
        ierr = cmor.set_grid_mapping(
            grid_id, 'polar_stereographic', param_dict)
        self.assertEqual(ierr, 0)


if __name__ == '__main__':
    run()



from __future__ import print_function
import cmor
import numpy
import unittest

# ==============================
#  main thread
# ==============================


def run():
    unittest.main()


ntimes = 1
nlat = 45
nlon = 90


class TestCase(unittest.TestCase):

    def setUp(self, *args, **kwargs):
        error_flag = cmor.setup(
            inpath='Tables',
            netcdf_file_action=cmor.CMOR_REPLACE)

        error_flag = cmor.dataset_json("Test/CMOR_input_example.json")

    def prep_var(self, var, units):
        # creates 1 degree grid
        dlat = 180 / nlat
        dlon = 360. / nlon
        alats = numpy.arange(-90 + dlat / 2., 90, dlat)
        bnds_lat = numpy.arange(-90, 90 + dlat, dlat)
        alons = numpy.arange(0 + dlon / 2., 360., dlon) - 180.
        bnds_lon = numpy.arange(0, 360. + dlon, dlon) - 180.
        cmor.load_table("Tables/CMIP6_Omon.json")
        # cmor.load_table("Test/IPCC_table_A1")
        ilat = cmor.axis(
            table_entry='latitude',
            units='degrees_north',
            length=nlat,
            coord_vals=alats,
            cell_bounds=bnds_lat)

        ilon = cmor.axis(
            table_entry='longitude',
            length=nlon,
            units='degrees_east',
            coord_vals=alons,
            cell_bounds=bnds_lon)

        itim = cmor.axis(
            table_entry='time',
            units='days since 2010-1-1')

        ivar1 = cmor.variable(
            var,
            axis_ids=[itim, ilat, ilon],
            units=units,
            missing_value=0.)

        return ivar1

    def testCMIP6(self):
        try:

            for d in range(2):
                mode = cmor.CMOR_APPEND
                if d == 0:
                    mode = cmor.CMOR_REPLACE
                ivar = self.prep_var("sos", "practical_salinity_scale_78")
                ivar2 = self.prep_var("sos", "psu")
                ivar3 = self.prep_var("sos", "pss-78")
                for i in range(4):
                    tval = [i / 4. + d]
                    tbnd = [i / 4. + d - 0.125, i / 4. + d + 0.125]
                    print('tvar', tval)
                    print('tbnd', tbnd)
                    print('writing time:', i, i / 4.)
                    data = numpy.random.random(
                        (ntimes, nlat, nlon)) * 30. + 273
                    data = data.astype("f")
                    cmor.write(ivar, data, time_vals=tval, time_bnds=tbnd)
                    tval = [i / 4. + d + 100]
                    tbnd = [i / 4. + d + 100 - 0.125, i / 4. + d + 100 + 0.125]
                    cmor.write(ivar2, data, time_vals=tval, time_bnds=tbnd)
                    tval = [i / 4. + d + 200]
                    tbnd = [i / 4. + d + 200 - 0.125, i / 4. + d + 200 + 0.125]
                    cmor.write(ivar3, data, time_vals=tval, time_bnds=tbnd)
                    print('wrote var 1 time:', i)
                file = cmor.close(ivar, True)
                file1 = cmor.close(ivar2, True)
                file2 = cmor.close(ivar3, True)
                print('File:', file)
                print('File:', file1)
                print('File:', file2)
        except BaseException:
            raise

    def tearDown(self):
        import shutil
        shutil.rmtree("./CMIP6")


if __name__ == '__main__':
    run()



#!/usr/bin/env python

'''
Note the memory size from repeated calls to cmor

Memory stats are taken from http://code.activestate.com/recipes/286222/
'''
from __future__ import print_function
import os

_proc_status = '/proc/%d/status' % os.getpid()

_scale = {'kB': 1024.0, 'mB': 1024.0 * 1024.0,
          'KB': 1024.0, 'MB': 1024.0 * 1024.0}


def _VmB(VmKey):
    '''Private.
    '''
    global _proc_status, _scale
    # get pseudo file  /proc/<pid>/status
    try:
        t = open(_proc_status)
        v = t.read()
        t.close()
    except BaseException:
        return 0.0  # non-Linux?
     # get VmKey line e.g. 'VmRSS:  9999  kB\n ...'
    i = v.index(VmKey)
    v = v[i:].split(None, 3)  # whitespace
    if len(v) < 3:
        return 0.0  # invalid format?
     # convert Vm value to bytes
    return float(v[1]) * _scale[v[2]]


def memory(since=0.0):
    '''Return memory usage in bytes.
    '''
    return _VmB('VmSize:') - since


def resident(since=0.0):
    '''Return resident memory usage in bytes.
    '''
    return _VmB('VmRSS:') - since


def stacksize(since=0.0):
    '''Return stack size in bytes.
    '''
    return _VmB('VmStk:') - since


import cmor
import sys


def memory_usage(fhd, mem=0, res=0, stk=0):
    nmem = memory()
    nres = resident()
    nstk = stacksize()
    fhd.write('memory: %s\t%i\t\t%i\t\t%i\n' %
              (nmem, memory(mem), resident(res), stacksize(stk)))
    return nmem, nres, nstk


def memory_check():
    cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_Amon.json'
    cmor.load_table(table)

    tval = []
    tbounds = []

    passtime = True
    ntimes = 1200
    for time in range(ntimes):
        tval.append(15 + time * 30)
        tbounds.append([time * 30, (time + 1) * 30])

    if passtime:
        timdef = {'table_entry': 'time',
                  'units': 'days since 2000-01-01 00:00:00',
                  }
    else:
        timdef = {'table_entry': 'time',
                  'units': 'days since 2000-01-01 00:00:00',
                  'coord_vals': tval,
                  'cell_bounds': tbounds,
                  }

    axes = [timdef,
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [[-1, 1], ],
             },
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)
    varid = cmor.variable('ts', 'K', axis_ids)

    mem = 0
    res = 0
    stk = 0
    memi, resi, stki = memory_usage(sys.stdout)

    for time in range(ntimes):
        a = [275]
        if passtime:
            tval = [15 + time * 30]
            tbounds = [time * 30, (time + 1) * 30]
            cmor.write(varid, [275], time_vals=tval, time_bnds=tbounds)
        else:
            cmor.write(varid, [275])
        mem, res, stk = memory_usage(sys.stdout, mem, res, stk)
        #mem,res,stk = memory_usage(sys.stdout,memi,resi,stki)
        print('---')

    cmor.close(varid)


if __name__ == '__main__':
    memory_check()



import numpy
# this test tries to mimic ippc_test_code.c but from python
# This one is using direct C calls from python not the python around it
ntimes = 2
lon = 4
lat = 3
lev = 5
lev2 = 19
varin3d = ["CLOUD", "U", "T"]

#  /* Units appropriate to my data */
units3d = ["%", "m s-1", "K"]

#  /* Corresponding IPCC Table A1c entry (variable name)  */
entry3d = ["cl", "ua", "ta"]

#  /* My variable names for IPCC Table A1a fields */
varin2d = ["LATENT", "TSURF", "SOIL_WET", "PSURF"]

#  /* Units appropriate to my data */
units2d = ["W m-2", "K", "kg m-2", "Pa"]

positive2d = ["down", " ", " ", " "]

#  /* Corresponding IPCC Table A1a entry (variable name)  */
entry2d = ["hfls", "tas", "mrsos", "ps"]


def gen_irreg_grid(lon, lat):
    lon0 = 280.
    lat0 = 0.
    delta_lon = 10.
    delta_lat = 10.
    y = numpy.arange(lat)
    x = numpy.arange(lon)
    lon_coords = numpy.zeros((lat, lon))
    lat_coords = numpy.zeros((lat, lon))
    lon_vertices = numpy.zeros((lat, lon, 4))
    lat_vertices = numpy.zeros((lat, lon, 4))

    for j in range(lat):  # really porr coding i know
        for i in range(lon):  # getting worse i know
            lon_coords[j, i] = lon0 + delta_lon * (j + 1 + i)
            lat_coords[j, i] = lat0 + delta_lat * (j + 1 - i)
            lon_vertices[j, i, 0] = lon_coords[j, i] - delta_lon
            lon_vertices[j, i, 1] = lon_coords[j, i]
            lon_vertices[j, i, 2] = lon_coords[j, i] + delta_lon
            lon_vertices[j, i, 3] = lon_coords[j, i]
# !!$      /* vertices lat */
            lat_vertices[j, i, 0] = lat_coords[j, i]
            lat_vertices[j, i, 1] = lat_coords[j, i] - delta_lat
            lat_vertices[j, i, 2] = lat_coords[j, i]
            lat_vertices[j, i, 3] = lat_coords[j, i] + delta_lat
    return x, y, lon_coords, lat_coords, lon_vertices, lat_vertices

# read_data funcs are highly unoptimzed....


def read_coords(lon, lat, lev):
    alons = numpy.zeros(lon)
    bnds_lon = numpy.zeros(2 * lon)
    alats = numpy.zeros(lat)
    bnds_lat = numpy.zeros(2 * lat)
    plevs = numpy.zeros(lev, dtype='i')
    for i in range(lon):
        alons[i] = i * 360. / lon
        bnds_lon[2 * i] = (i - 0.5) * 360. / lon
        bnds_lon[2 * i + 1] = (i + 0.5) * 360. / lon

    for i in range(lat):
        alats[i] = (lat - i) * 10
        bnds_lat[2 * i] = (lat - i) * 10 + 5.
        bnds_lat[2 * i + 1] = (lat - i) * 10 - 5.

    plevs = numpy.array([100000., 92500., 85000., 70000.,
                         60000., 50000., 40000., 30000., 25000., 20000.,
                         15000., 10000., 7000., 5000., 3000., 2000., 1000., 500., 100.])

    return alats, alons, plevs, bnds_lat, bnds_lon


def read_time(it):
    time = [0]
    time_bnds = [0, 0]
    time[0] = (it - 0.5) * 30.
    time_bnds[0] = (it - 1) * 30.
    time_bnds[1] = it * 30.

    time[0] = it
    time_bnds[0] = it
    time_bnds[1] = it + 1
    return time[0], numpy.array(time_bnds)


def read_3d_input_files(it, varname, n0, n1, n2, ntimes):

    if varname == "CLOUD":
        factor = 0.1
        offset = -50.
    elif varname == "U":
        factor = 1.
        offset = 100.
    elif varname == "T":
        factor = 0.5
        offset = -150.

    field = numpy.zeros((n2, n1, n0), dtype='d')
    for k in range(n2):
        for j in range(n1):
            for i in range(n0):
                field[k, j, i] = (k * 64 + j * 16 + i *
                                  4 + it) * factor - offset
    return field


def read_2d_input_files(it, varname, n0, n1):

    if varname == "LATENT":
        factor = 1.25
        offset = 100.
    elif varname == "TSURF":
        factor = 2.0
        offset = -230.
    elif varname == "SOIL_WET":
        factor = 10.
        offset = 0.
    elif varname == "PSURF":
        factor = 1.
        offset = -9.7e2

    field = numpy.zeros((n0, n1), dtype='d')

    for j in range(n0):
        for i in range(n1):
            tmp = (j * 16. + i * 4. + it) * factor - offset
            field[j, i] = tmp
    return field


alats, alons, plevs, bnds_lat, bnds_lon = read_coords(lon, lat, lev)

Time = numpy.zeros(ntimes, dtype='d')
bnds_time = numpy.zeros(ntimes * 2, dtype='d')
Time[0], bnds_time[0:2] = read_time(0)
Time[1], bnds_time[2:4] = read_time(1)

zlevs = numpy.zeros(5, dtype='d')
zlevs[0] = 0.1999999999999999999
zlevs[1] = 0.3
zlevs[2] = 0.55
zlevs[3] = 0.7
zlevs[4] = 0.99999999

zlev_bnds = numpy.zeros(6, dtype='d')
zlev_bnds[0] = 0.
zlev_bnds[1] = 0.2
zlev_bnds[2] = 0.42
zlev_bnds[3] = 0.62
zlev_bnds[4] = 0.8
zlev_bnds[5] = 1.

regions = numpy.array(["atlantic_arctic_ocean",
                       "indian_pacific_ocean",
                       "pacific_ocean",
                       "global_ocean",
                       "sf_bay"])

a_coeff = numpy.array([0.1, 0.2, 0.3, 0.22, 0.1])
b_coeff = numpy.array([0.0, 0.1, 0.2, 0.5, 0.8])
p0 = numpy.array([1.e5, ])
a_coeff_bnds = numpy.array([0., .15, .25, .25, .16, 0.])
b_coeff_bnds = numpy.array([0., .05, .15, .35, .65, 1.])



from __future__ import print_function
from test_python_common import *  # common subroutines

import cmor._cmor
import os

pth = os.path.split(os.path.realpath(os.curdir))
if pth[-1] == 'Test':
    ipth = opth = '.'
else:
    ipth = opth = 'Test'

myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


cmor.setup(
    inpath="Tables",
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
# cmor.dataset_json("Test/CMOR_input_example.json")
cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
a = cmor.load_table(os.path.join("CMIP6_Omon.json"))
tables.append(a)
tables.append(cmor.load_table("CMIP6_Amon.json"))
print('Tables ids:', tables)

axes = []
id = "time"
units = "months since 1980"
print('time bounds:', bnds_time)
# ok we need to make the bounds 2D because the cmor module "undoes this"
bnds_time = numpy.reshape(bnds_time, (int(bnds_time.shape[0] / 2), 2))
bnds_lat = numpy.reshape(bnds_lat, (int(bnds_lat.shape[0] / 2), 2))
bnds_lon = numpy.reshape(bnds_lon, (int(bnds_lon.shape[0] / 2), 2))
myaxes[0] = cmor.axis(
    id,
    coord_vals=Time,
    units=units,
    cell_bounds=bnds_time,
    interval="1 month")
print('time bounds:', bnds_time)
id = 'latitude'
units = "degrees_north"
interval = ""
myaxes[1] = cmor.axis(id, coord_vals=alats, units=units, cell_bounds=bnds_lat)
id = "longitude"
units = "degrees_east"
myaxes[2] = cmor.axis(id, coord_vals=alons, units=units, cell_bounds=bnds_lon)
id = "plev19"
units = "Pa"
myaxes[3] = cmor.axis(id, coord_vals=plevs, units=units)

myaxes[4] = cmor.axis(
    "alternate_hybrid_sigma",
    coord_vals=zlevs,
    units="1",
    cell_bounds=zlev_bnds)


cmor.set_table(tables[0])

myaxes[5] = cmor.axis("basin", coord_vals=regions, units="")
id = 'time'
units = 'months since 1980'
myaxes[7] = cmor.axis(
    id,
    coord_vals=Time,
    units=units,
    cell_bounds=bnds_time,
    interval="1 month")
id = "latitude"
units = "degrees_north"
interval = ""
myaxes[8] = cmor.axis(id, coord_vals=alats, units=units, cell_bounds=bnds_lat)

cmor.set_table(tables[1])


dtmp = -999
dtmp2 = 1.e-4
myaxes2[0] = myaxes[0]
myaxes2[1] = myaxes[3]
myaxes2[2] = myaxes[1]
myaxes2[3] = myaxes[2]

print('ok doing the vars thing', positive2d[0])
myvars[0] = cmor.variable(entry2d[0],
                          units2d[0],
                          myaxes[:3],
                          'd',
                          missing_value=None,
                          tolerance=dtmp2,
                          positive=positive2d[0],
                          original_name=varin2d[0],
                          history="no history",
                          comment="no future")
print('vars 2')
myvars[1] = cmor.variable(entry3d[2], units3d[2],
                          myaxes2[:4], 'd', original_name=varin3d[2])
print('vars 2')

myaxes2[1] = myaxes[4]
myvars[2] = cmor.variable(entry3d[0], units3d[0],
                          myaxes2[:4], 'd', original_name=varin3d[0])

print('vars 2')


myvars[3] = cmor.zfactor(int(myaxes2[1]), "p0", "Pa", None, 'd', p0)
print('zfact', myaxes2[1])
myvars[3] = cmor.zfactor(int(myaxes2[1]), "b", "",
                         myaxes2[1], 'd', b_coeff, b_coeff_bnds)
print('zfact', myaxes2[1])
myvars[3] = cmor.zfactor(int(myaxes2[1]), "a", "",
                         myaxes2[1], 'd', a_coeff, a_coeff_bnds)
#/*   printf("defining ap\n"); */
#/*   for(i=0;i<5;i++) {a_coeff[i]*=1.e3;printf("sending acoef: %i, %lf\n",i,a_coeff[i]);} */
#/*   for(i=0;i<6;i++) {a_coeff_bnds[i]*=1.e5;printf("sending acoef: %i, %lf\n",i,a_coeff_bnds[i]);} */
#/*   ierr = cmor_zfactor(&myvars[3],myaxes2[1],"ap","hPa",1,&myaxes2[1],'d',&a_coeff,&a_coeff_bnds); */
print('zfact before last')
myvars[3] = cmor.zfactor(zaxis_id=myaxes2[1],
                         zfactor_name="ps",
                         units="hPa",
                         axis_ids=myaxes[:3],
                         type='d')
print('zfact last')

#  /* ok here we decalre a variable for region axis testing */
cmor.set_table(tables[0])
myaxes2[0] = myaxes[7]
myaxes2[1] = myaxes[5]
myaxes2[2] = myaxes[8]

myvars[4] = cmor.variable("htovgyre",
                          "W",
                          myaxes2[:3],
                          'd',
                          positive=positive2d[0],
                          original_name=varin2d[0])

cmor.set_table(tables[1])


for i in range(ntimes):
    data2d = read_2d_input_files(i, varin2d[0], lat, lon)
    print('writing time: ', i, data2d.shape, data2d)
    cmor.write(myvars[0], data2d, 1)

cmor.close()



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import sys
import json
import pdb
import requests
from collections import OrderedDict


# List of files needed from github for CMIP6 CV
# ---------------------------------------------
filelist = [ 
        "CMIP6_required_global_attributes.json",
        "CMIP6_activity_id.json",
        "CMIP6_institution_id.json",
        "CMIP6_source_id.json",
        "CMIP6_source_type.json",
        "CMIP6_frequency.json",
        "CMIP6_grid_label.json",
        "CMIP6_nominal_resolution.json",
        "CMIP6_realm.json",
        "CMIP6_table_id.json",
        "CMIP6_license.json",
        "mip_era.json",
        "CMIP6_sub_experiment_id.json",
        "CMIP6_experiment_id.json"
        ]
# Github repository with CMIP6 related Control Vocabulary files
# -------------------------------------------------------------
githubRepo = "https://raw.githubusercontent.com/WCRP-CMIP/CMIP6_CVs/master/"

class readWCRP():
    def __init__(self):
        pass

    def createSource(self,myjson):
        root = myjson['source_id']
        for key in root.keys():
            root[key]['source']=root[key]['label'] + ' (' + root[key]['release_year'] + '): ' + chr(10)
            for realm in root[key]['model_component'].keys():
                if( root[key]['model_component'][realm]['description'].find('None') == -1):
                    root[key]['source'] += realm + ': ' 
                    root[key]['source'] += root[key]['model_component'][realm]['description'] + chr(10)
            root[key]['source'] = root[key]['source'].rstrip()
            del root[key]['label']
            del root[key]['release_year']
            del root[key]['label_extended']
            del root[key]['model_component']

    def createExperimentID(self,myjson):
        #
        # Delete undesirable attribute for experiement_id
        #
        root = myjson['experiment_id']
        for key in root.keys():
            del root[key]['tier']
            del root[key]['start_year']
            del root[key]['end_year']
            del root[key]['description']
            del root[key]['min_number_yrs_per_sim']

    def readGit(self):
        Dico = OrderedDict()
        for file in filelist:
            url = githubRepo + file 
            response = requests.get(url)
            print(url)
            urlJson = response.content.decode('utf-8')
            myjson = json.loads(urlJson, object_pairs_hook=OrderedDict)
            if(file == 'CMIP6_source_id.json'):
                self.createSource(myjson)
            if(file == 'CMIP6_experiment_id.json'):
                self.createExperimentID(myjson)
            Dico = OrderedDict(Dico.items() + myjson.items())
         
        finalDico = OrderedDict()
        finalDico['CV'] = Dico
        return finalDico

def run():
    f = open("CMIP6_CV.json", "w")
    gather = readWCRP()
    CV = gather.readGit()
    regexp = {}
    regexp["variant_label"] = ["r[[:digit:]]\\{1,\\}i[[:digit:]]\\{1,\\}p[[:digit:]]\\{1,\\}f[[:digit:]]\\{1,\\}$" ]

    regexp["tracking_id"] = [ "hdl:21.14100/.*" ]  
    regexp["mip_era"] = [ "CMIP6" ]
    regexp["further_info_url"] = [ "https://furtherinfo.es-doc.org/.*" ]
    regexp["product"] = [ "model-output" ]
    regexp["Conventions"] = [ "^CF-1.7 CMIP-6.[0-2]\\( UGRID-1.0\\)\\{0,\\}$" ]
    regexp["realization_index"] = [ "^\\[\\{0,\\}[[:digit:]]\\{1,\\}\\]\\{0,\\}$" ]
    regexp["physics_index"] = [ "^\\[\\{0,\\}[[:digit:]]\\{1,\\}\\]\\{0,\\}$" ]
    regexp["forcing_index"] = [ "^\\[\\{0,\\}[[:digit:]]\\{1,\\}\\]\\{0,\\}$" ]
    regexp["initialization_index"] = [ "^\\[\\{0,\\}[[:digit:]]\\{1,\\}\\]\\{0,\\}$" ]
    regexp["data_specs_version"] = [ "^[[:digit:]]\\{2,2\\}\\.[[:digit:]]\\{2,2\\}\\.[[:digit:]]\\{2,2\\}$" ]
    regexp["license"] = [ "^CMIP6 model data produced by .* is licensed under a Creative Commons Attribution.*ShareAlike 4.0 International License .https://creativecommons.org/licenses.* *Consult https://pcmdi.llnl.gov/CMIP6/TermsOfUse for terms of use governing CMIP6 output, including citation requirements and proper acknowledgment\\. *Further information about this data, including some limitations, can be found via the further_info_url (recorded as a global attribute in this file).*\\. *The data producers and data providers make no warranty, either express or implied, including, but not limited to, warranties of merchantability and fitness for a particular purpose\\. *All liabilities arising from the supply of the information (including any liability arising in negligence) are excluded to the fullest extent permitted by law\\.$" ]


    CV['CV'] = OrderedDict(CV['CV'].items() + regexp.items())
    for exp in CV["CV"]["experiment_id"]:
        CV["CV"]["experiment_id"][exp]["activity_id"] = [ " ".join(CV["CV"]["experiment_id"][exp]["activity_id"])]
        print("AC ID:",CV["CV"]["experiment_id"][exp]["activity_id"])
    f.write(json.dumps(CV, indent=4, separators=(',', ':'), sort_keys=False) )

    f.close()

if __name__ == '__main__':
    run()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import sys
import os
import tempfile


class TestCase(unittest.TestCase):

    def testCMIP6(self):

        # ------------------------------------------------------
        # Copy stdout and stderr file descriptor for cmor output
        # ------------------------------------------------------
        newstdout = os.dup(1)
        newstderr = os.dup(2)
        # --------------
        # Create tmpfile
        # --------------
        tmpfile = tempfile.mkstemp()
        os.dup2(tmpfile[0], 1)
        os.dup2(tmpfile[0], 2)
        os.close(tmpfile[0])
        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        try:
            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
            cmor.dataset_json("Test/CMOR_input_example.json")
            cmor.set_cur_dataset_attribute("experiment_id", "ssp434")
            cmor.set_cur_dataset_attribute(
                "parent_experiment_id", "historical")
            cmor.set_cur_dataset_attribute("parent_activity_id", "CMIP")
            cmor.set_cur_dataset_attribute("activity_id", "ScenarioMIP")
            cmor.set_cur_dataset_attribute("source_type", "AOGCM")
            cmor.set_cur_dataset_attribute("sub_experiment_id", "none")
            cmor.set_cur_dataset_attribute(
                "parent_variant_label", "r11i123p4556f333")
            cmor.set_cur_dataset_attribute("parent_source_id", "child")
            cmor.set_cur_dataset_attribute("parent_mip_era", "CMIP6")

            # ------------------------------------------
            # load Omon table and create masso variable
            # ------------------------------------------
            cmor.load_table("CMIP6_Omon.json")
            cmor.load_table("CMIP6_Amon.json")
            cmor.load_table("CMIP6_6hrPlev.json")
            cmor.load_table("CMIP6_6hrPlevPt.json")
            cmor.load_table("CMIP6_AERday.json")
            cmor.load_table("CMIP6_AERfx.json")
            cmor.load_table("CMIP6_AERhr.json")
            cmor.load_table("CMIP6_AERmon.json")
            cmor.load_table("CMIP6_AERmonZ.json")
            cmor.load_table("CMIP6_Amon.json")
            cmor.load_table("CMIP6_CFday.json")
            cmor.load_table("CMIP6_CFmon.json")
            cmor.load_table("CMIP6_CFsubhr.json")
            cmor.load_table("CMIP6_CFsubhrOff.json")
            cmor.load_table("CMIP6_day.json")
            cmor.load_table("CMIP6_E1hrClimMon.json")
            cmor.load_table("CMIP6_E1hr.json")
            cmor.load_table("CMIP6_E3hr.json")
            cmor.load_table("CMIP6_E6hrZ.json")
            cmor.load_table("CMIP6_EdayZ.json")
            cmor.load_table("CMIP6_Efx.json")
            cmor.load_table("CMIP6_EmonZ.json")
            cmor.load_table("CMIP6_Esubhr.json")
            cmor.load_table("CMIP6_Eyr.json")
            cmor.load_table("CMIP6_fx.json")
            cmor.load_table("CMIP6_grids.json")
        except BaseException:
            pass
        os.dup2(newstdout, 1)
        os.dup2(newstderr, 2)
        sys.stdout = os.fdopen(newstdout, 'wb', 0)
        sys.stderr = os.fdopen(newstderr, 'wb', 0)
        f = open(tmpfile[1], 'r')
        lines = f.readlines()
        for line in lines:
            if line.find('Error:') != -1:
                self.assertIn('30', line.strip())
                break
        f.close()
        os.unlink(tmpfile[1])

    # def tearDown(self):
    #     import shutil
    #     shutil.rmtree("./CMIP6")


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
from test_python_common import *  # common subroutines
import cmor._cmor
import os
pth = os.path.split(os.path.realpath(os.curdir))
if pth[-1] == 'Test':
    ipth = opth = '.'
else:
    ipth = opth = 'Test'

myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')

cmor._cmor.setup(
    ipth,
    cmor.CMOR_REPLACE,
    cmor.CMOR_NORMAL,
    cmor.CMOR_EXIT_ON_MAJOR,
    "",
    1)
cmor.dataset_json("Test/CMOR_input_example.json")
# cmor.dataset_json("Test/CMOR_input_example.json")

tables = []
a = cmor._cmor.load_table("Tables/CMIP6_Omon.json")
tables.append(a)
tables.append(cmor._cmor.load_table("Tables/CMIP6_Amon.json"))
print('Tables ids:', tables)


axes = []
id = "time"
units = "months since 1980"
myaxes[0] = cmor._cmor.axis(
    id,
    units,
    ntimes,
    Time,
    b'd',
    bnds_time,
    2,
    "1 month")
id = 'latitude'
units = "degrees_north"
interval = ""
myaxes[1] = cmor._cmor.axis(id, units, lat, alats, b'd', bnds_lat, 2, interval)
id = "longitude"
units = "degrees_east"
myaxes[2] = cmor._cmor.axis(id, units, lon, alons, b'd', bnds_lon, 2, interval)
id = "plev19"
units = "Pa"
print(plevs.astype("d"))
myaxes[3] = cmor._cmor.axis(
    id,
    units,
    lev2,
    plevs.astype("d"),
    b'd',
    None,
    0,
    interval)


myaxes[4] = cmor._cmor.axis(
    "alternate_hybrid_sigma",
    "1",
    5,
    zlevs,
    b'd',
    zlev_bnds,
    1,
    interval)


cmor.set_table(tables[0])
myaxes[5] = cmor._cmor.axis("basin", "", 4, regions, b'c', None, 21, interval)
id = 'time'
units = 'months since 1980'
myaxes[7] = cmor._cmor.axis(
    id,
    units,
    ntimes,
    Time,
    b'd',
    bnds_time,
    2,
    "1 month")
id = "latitude"
units = "degrees_north"
interval = ""
myaxes[8] = cmor._cmor.axis(id, units, lat, alats, b'd', bnds_lat, 2, interval)

cmor._cmor.set_table(tables[1])


dtmp = -999
dtmp2 = 1.e-4
myaxes2[0] = myaxes[0]
myaxes2[1] = myaxes[3]
myaxes2[2] = myaxes[1]
myaxes2[3] = myaxes[2]

print('ok doing the vars thing')
myvars[0] = cmor._cmor.variable(
    entry2d[0],
    units2d[0],
    3,
    myaxes,
    b'd',
    None,
    dtmp2,
    positive2d[0],
    varin2d[0],
    "no history",
    "no future")
print('vars 2')
myvars[1] = cmor._cmor.variable(
    entry3d[2],
    units3d[2],
    4,
    myaxes2,
    b'd',
    None,
    dtmp2,
    "down",
    varin3d[2],
    "no history",
    "no future")
print('vars 2')

myaxes2[1] = myaxes[4]
myvars[2] = cmor._cmor.variable(
    entry3d[0],
    units3d[0],
    4,
    myaxes2,
    b'd',
    None,
    dtmp2,
    "down",
    varin3d[0],
    "no history",
    "no future")

print('vars 2')

print('zfact', type(numpy.array(myaxes2[1])), type(myaxes2))

myvars[3] = cmor._cmor.zfactor(
    int(myaxes2[1]), "p0", "Pa", 0, None, b'd', p0, None)
print('zfact', myaxes2[1])
myvars[3] = cmor._cmor.zfactor(
    int(myaxes2[1]), "b", "", 1, myaxes2[1], b'd', b_coeff, b_coeff_bnds)
print('zfact', myaxes2[1])
myvars[3] = cmor._cmor.zfactor(
    int(myaxes2[1]), "a", "", 1, myaxes2[1], b'd', a_coeff, a_coeff_bnds)
#/*   printf("defining ap\n"); */
#/*   for(i=0;i<5;i++) {a_coeff[i]*=1.e3;printf("sending acoef: %i, %lf\n",i,a_coeff[i]);} */
#/*   for(i=0;i<6;i++) {a_coeff_bnds[i]*=1.e5;printf("sending acoef: %i, %lf\n",i,a_coeff_bnds[i]);} */
#/*   ierr = cmor_zfactor(&myvars[3],myaxes2[1],"ap","hPa",1,&myaxes2[1],'d',&a_coeff,&a_coeff_bnds); */
print('zfact before last')
myvars[3] = cmor._cmor.zfactor(
    int(myaxes2[1]), "ps", "hPa", 3, myaxes, b'd', None, None)
print('zfact last')

#  /* ok here we decalre a variable for region axis testing */
cmor.set_table(tables[0])
myaxes2[0] = myaxes[7]
myaxes2[1] = myaxes[5]
myaxes2[2] = myaxes[8]

myvars[4] = cmor._cmor.variable(
    "htovgyre",
    "W",
    3,
    myaxes2,
    b'd',
    None,
    dtmp2,
    positive2d[0],
    varin2d[0],
    "no history",
    "no future")

cmor.set_table(tables[1])


for i in range(ntimes):
    data2d = read_2d_input_files(i, varin2d[0], lat, lon)
    print('writing time', i, data2d.shape, data2d, numpy.average(data2d))
    cmor._cmor.write(
        myvars[0],
        numpy.ravel(data2d),
        data2d.dtype.char,
        "",
        1,
        None,
        None,
        None)

cmor._cmor.close(None, 0, 0)



import cmor
import numpy


def cmor_initialisation():
    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset_json("Test/CMOR_input_example.json")


def setup_data():
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00'
             },
            {'table_entry': 'latitude',
             'units': 'degrees',
             'coord_vals': [0],
             'cell_bounds': [-0.5, 0.5]},
            {'table_entry': 'longitude',
             'units': 'degrees',
             'coord_vals': [361., 362.],
             'cell_bounds': [360., 361., 362.]},
            ]

    values = numpy.array([215., 216.], numpy.float32)
    return values, axes


def cmor_define_and_write(values, axes):
    table = 'CMIP6_Amon.json'
    cmor.load_table(table)

    axis_ids = list()
    for axis in axes:
        axis_ids.append(cmor.axis(**axis))

    table = 'CMIP6_Amon.json'
    cmor.load_table(table)

    varid = cmor.variable('rlut',
                          'W m-2',
                          axis_ids,
                          history='variable history',
                          missing_value=-99,
                          positive='up'
                          )

    cmor.write(varid, values, time_vals=[15], time_bnds=[0, 30])


def version(cmor):
    return '%s.%s.%s' % (cmor.CMOR_VERSION_MAJOR,
                         cmor.CMOR_VERSION_MINOR,
                         cmor.CMOR_VERSION_PATCH)


def main():
    assert version(cmor) >= '3.0.0'
    cmor_initialisation()
    values, axes = setup_data()
    cmor_define_and_write(values, axes)
    cmor.close()


if __name__ == '__main__':

    main()



#!/usr/bin/env python

from __future__ import print_function
import cdms2
import sys
import genutil
cdms2.setAutoBounds('on')

type = 'd'

order = 'zxty'
# order='xty'
#order = None
#order= 'txy'
var = 'ta'
if len(sys.argv) > 2:
    fnm = sys.argv[1]
    fout = sys.argv[2]
else:
    fnm = '/ipcc/20c3m/atm/mo/%s/ncar_ccsm3_0/run1/%s_A1.20C3M_1.CCSM.atmm.1870-01_cat_1879-12.nc' % (
        var, var)
    fout = 'Test/%s.asc' % (var)

f = cdms2.open(fnm)

ntimes = 3
print('var:', var)
# s=f(var,time=slice(0,3),latitude=(-20,20),order=order,squeeze=1)
if order is not None:
    if order.find('z') > -1:
        s = f(var, time=slice(0, ntimes), order=order, squeeze=1,
              longitude=(-180, 180, 'con'), level=slice(5, 12))
        print(s.getLevel()[:])
    else:
        s = f(var, time=slice(0, ntimes), order=order,
              squeeze=1, longitude=(-180, 180, 'con'))
else:
    s = f(var, time=slice(0, ntimes), squeeze=1)
# s=s[:,::4,::4]
print('Read in', s.shape)
try:
    p = s.getLevel()
    p.id = 'pressure'
    p.units = 'Pa'
except BaseException:
    pass
# print genutil.minmax(s)
f.close()

f = open(fout, 'w')

ndim = s.rank()
print('Dumping')
print(s.id, file=f)
print(s.units, file=f)
print(ndim, file=f)

for i in range(ndim):
    ax = s.getAxis(i)
    print(len(ax), file=f)

for i in range(ndim):
    ax = s.getAxis(i)
    if ax.isLatitude():
        print('latitude', file=f)
    elif ax.isLongitude():
        print('longitude', file=f)
    else:
        print(ax.id, file=f)
    print(ax.units, file=f)
    print(ax.id)
    print(ax.units)
    for j in ax[:]:
        print(j, end=' ', file=f)
    print(file=f)
    for j in ax.getBounds().flat[:]:
        print(j, end=' ', file=f)
    print(file=f)
f.flush()

s = s.filled(120).astype(type)
s = s.flat
j = 0
for i in s[:]:
    print(i, end=' ', file=f)
    j += 1
print(file=f)
print(j, s[-1])

f.close()
